<?php

namespace App\Services\Approval;

use App\Models\Approval;
use App\Models\ApprovalFlow;
use App\Models\Enums\ApprovalFlowObjectType;
use App\Models\Enums\ApprovalFlowType;
use App\Models\Enums\ApprovalMode;
use App\Models\Enums\ApprovalObjectType;
use App\Models\Enums\ObjectAction;
use App\Models\Enums\ObjectApprovalStatus;
use App\Models\File;
use App\Repositories\ActionRepository;
use Illuminate\Support\Facades\Auth;

class StoreService
{
    public function store()
    {
        $request = request();
        $approvalObjectType = ApprovalObjectType::from($request->object_type);

        $approvalObject = $approvalObjectType->modelBuilderAllowed($request->object_id)->findOrFail($request->object_id);


        $this->checkStatus($approvalObject, $approvalObjectType);

//        $approvalFlow = $this->getApprovalFlow($approvalObjectType, $approvalObject);
        $approvalFlow = $this->getNewApprovalFlow($request->approvalFlow_id);

        $containerFileIds = $request->get("file_ids", []);

        $this->createApproval($approvalFlow, $approvalObjectType, $approvalObject, $request->remark, $containerFileIds);
    }

    protected function createApproval(
        ApprovalFlow $approvalFlow,
        ApprovalObjectType $approvalObjectType,
        $approvalObject,
        string $remark = null,
        array $containerFileIds = [],
    ): void
    {
        $firstNode =  $approvalFlow->nodes[1];

        $approval = Approval::query()->create([
            'approval_flow_id' => $approvalFlow->id,
            'object_type' => $approvalObjectType->flowType()->value,
            'object_id' => $approvalObject->id,
            'node_level' => $firstNode['level'],
            'approval_mode' => $firstNode['approval_mode'],
            'users' => sprintf(",%s,", implode(',', $firstNode['approval_users'])),
            'remark' => $remark,
            'created_by' => Auth::id(),
            'status' => ObjectApprovalStatus::DOING,
            'company_id' => Auth::user()->company_id,
            'sub_object_ids' => $containerFileIds,
        ]);

        $extraFields = [
            'now' => 0,
            'next' => 1,
            'approval_mode' => $firstNode['approval_mode'],
        ];

        if ($approval->approval_mode == ApprovalMode::SEQUENTIAL_APPROVAL->value) {
            $extraFields['next_sequential_user'] = $firstNode['approval_users'][0];
        }

        ActionRepository::createByApproval(
            $approval,
            ObjectAction::APPROVAL_REQUEST,
            extraFields:  $extraFields,
        );

        $this->changeApprovalStatus($approvalObject, $approvalObjectType, $approval, $containerFileIds);
    }

    protected function checkStatus($approvalObject, ApprovalObjectType $approvalObjectType)
    {
        if ($approvalObjectType == ApprovalObjectType::CONTAINER_FILE) {
            return;
        }

        throw_validation_if(
            in_array($approvalObject?->approval_status, [ObjectApprovalStatus::DOING->value]),
            'Do not initiate duplicate approvals'
        );
    }

    protected function changeApprovalStatus(
        $approvalObject, ApprovalObjectType $approvalObjectType, Approval $approval, array $containerFileIds = []
    ): void
    {
        if ($approvalObjectType == ApprovalObjectType::CONTAINER_FILE) {
            File::query()->whereIn("id", $containerFileIds)->update([
                'approval_status' => ObjectApprovalStatus::DOING,
                'latest_approval_id' => $approval->id,
            ]);
            return;
        }

        $approvalObject->approval_status = ObjectApprovalStatus::DOING;
        $approvalObject->save();
    }

    protected function getApprovalFlow(ApprovalObjectType $approvalObjectType, $approvalObject)
    {
        $approvalFlow = ApprovalFlow::query()
            ->where(function ($query) use ($approvalObjectType, $approvalObject) {
                $query->where([
                    "type" => $approvalObjectType->flowType()->value,
                    "object_id" => 0,
                ])
                    ->when(
                        $approvalObjectType->flowType() == ApprovalFlowType::TASK,
                        fn ($query) => $query->orWhere(fn ($query) => $query->where([
                            "type" => $approvalObjectType->flowType()->value,
                            'object_type' => ApprovalFlowObjectType::PROJECT->value,
                            'object_id' => $approvalObject->project_id,
                        ]))
                    );
            })
            ->where("status", 1)
            ->orderByDesc("object_id")
            ->first();
        throw_validation_if(! $approvalFlow, 'Clearly set up approval flow rules first.');

        return $approvalFlow;
    }

    protected function getNewApprovalFlow($id)
    {
        $approvalFlow = ApprovalFlow::query() ->where("status", 1)->findOrFail($id);

        throw_validation_if(! $approvalFlow, 'Clearly set up approval flow rules first.');

        return $approvalFlow;
    }


}
